//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/MesoCrystalItem.h
//! @brief     Defines class MesoCrystalItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_MESOCRYSTALITEM_H
#define BORNAGAIN_GUI_MODELS_MESOCRYSTALITEM_H

#include "Base/Vector/Vectors3D.h"
#include "GUI/Models/ItemWithParticles.h"
#include "Sample/Lattice/Lattice3D.h"

class FormFactorItem;
class IFormFactor;
class IParticle;
class MesoCrystal;
class VectorItem;

class BA_CORE_API_ MesoCrystalItem : public ItemWithParticles {
private:
    static const QString P_OUTER_SHAPE;
    static const QString T_BASIS_PARTICLE;
    static const QString P_VECTOR_A;
    static const QString P_VECTOR_B;
    static const QString P_VECTOR_C;

public:
    static const QString M_TYPE;

    MesoCrystalItem();

    std::unique_ptr<MesoCrystal> createMesoCrystal() const;

    Lattice3D getLattice() const;
    std::unique_ptr<IParticle> getBasis() const;

    std::unique_ptr<IFormFactor> getOuterShape() const;
    template <typename T> T* setOuterShapeType();

    ItemWithParticles* basisParticle() const;

    void setVectorA(const kvector_t& vector_a);
    void setVectorB(const kvector_t& vector_b);
    void setVectorC(const kvector_t& vector_c);
};

template <typename T> T* MesoCrystalItem::setOuterShapeType()
{
    static_assert(std::is_base_of<FormFactorItem, T>::value,
                  "Class must be derived from FormFactorItem");

    return setGroupPropertyType<T>(P_OUTER_SHAPE);
}

#endif // BORNAGAIN_GUI_MODELS_MESOCRYSTALITEM_H
