//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/widgetbox/widgetboxcategorylistview.h
//! @brief     Defines class WidgetBoxCategoryListView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_WIDGETBOX_WIDGETBOXCATEGORYLISTVIEW_H
#define BORNAGAIN_GUI_VIEWS_WIDGETBOX_WIDGETBOXCATEGORYLISTVIEW_H

#include <QListView>

class QSortFilterProxyModel;
class WidgetBoxCategoryModel;

// List view of a category, switchable between icon and list mode.
// Provides a filtered view.
class WidgetBoxCategoryListView : public QListView {
    Q_OBJECT
public:
    // Whether to access the filtered or unfiltered view
    enum AccessMode { FILTERED, UNFILTERED };

    explicit WidgetBoxCategoryListView(QWidget* parent = 0);
    void setViewMode(ViewMode vm);

    using QListView::contentsSize;

    // These method operates on the filtered/unfiltered model according to accessmode
    int count(AccessMode accessmode) const;

    void addEntry(const QString& title, const QIcon& icon, const QString& className,
                  const QString& tooltip);

signals:
    void pressed(const QString& name, const QString& className, const QPoint& globalPos);

public slots:
    void filter(const QRegExp& re);

private slots:
    void slotPressed(const QModelIndex& index);

private:
    int mapRowToSource(int filterRow) const;
    QSortFilterProxyModel* m_proxyModel;
    WidgetBoxCategoryModel* m_model;
};

#endif // BORNAGAIN_GUI_VIEWS_WIDGETBOX_WIDGETBOXCATEGORYLISTVIEW_H
